/*
 * Copyright 2006 Eric Bodden
 */
package ca.mcgill.sable.ft.reccache.cache;

import org.aspectj.lang.reflect.FieldSignature;

/**
 * A field ID uniquely identifies a field.
 * It consists of an owner object and a field signature.
 * For static fields, the owner object is the class that owns the field.
 *
 * @author Eric Bodden
 */
public class FieldId {
	
	/** the owner object */
	Object o;
	
	/** the field signature */
	FieldSignature s;
	
	/**
	 * Creates a new field id.
	 * @param o the owner object
	 * @param s the field signature
	 */
	public FieldId(Object o, FieldSignature s) {
		this.o = o;
		this.s = s;
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public int hashCode() {
		final int PRIME = 31;
		int result = 1;
		result = PRIME * result + ((o == null) ? 0 : o.hashCode());
		result = PRIME * result + ((s == null) ? 0 : s.hashCode());
		return result;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final FieldId other = (FieldId) obj;
		if (o == null) {
			if (other.o != null)
				return false;
		} else if (!o.equals(other.o))
			return false;
		if (s == null) {
			if (other.s != null)
				return false;
		} else if (!s.equals(other.s))
			return false;
		return true;
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		return "<"+o+","+s+">";
	}

}