/*
 * Copyright 2006 Eric Bodden
 */
package ca.mcgill.sable.ft.reccache.interception;

import org.aspectj.lang.annotation.SuppressAjWarnings;

import ca.mcgill.sable.ft.reccache.annotation.Recoverable;
import ca.mcgill.sable.ft.reccache.cache.Checkpoints;

/**
 * An aspect that implements recoverable blocks using checkpointing. 
 *
 * @author Eric Bodden
 */
public aspect Recoverables percflow(recoverable()){
	
	declare soft : NoSuchFieldException : execution(* Recoverables.currentValue(..));
	
	/** matches the execution of any recoverable method */
	pointcut recoverable(): execution(@Recoverable * *(..));

	/**
	 * before entring a recoverable region, establish a checkpoint
	 */
	@SuppressAjWarnings("adviceDidNotMatch")
	before(): recoverable() {
		Checkpoints.establish();
	}
	
	/**
	 * when leaving a recoverable region by return (i.e. no exception is thrown),
	 * we discard the checkpoint
	 */
	@SuppressAjWarnings("adviceDidNotMatch")
	after() returning: recoverable() {
		Checkpoints.discard();
	} 

	/**
	 * when leaving a recoverable region by exception,
	 * we restore the state from the last checkpoint
	 */
	@SuppressAjWarnings("adviceDidNotMatch")
	after() throwing: recoverable() {
		Checkpoints.restore();
	} 
	
}
